@@ -1,4 +1,4 @@
-# Author: Murat Uenalan (muenalan@cpan.org)
+# Author: Murat Uenalan (murat.uenalan@charite.de)
 #
 # Copyright (c) 2001 Murat Uenalan. All rights reserved.
 #
@@ -8,375 +8,219 @@
 
 package Class::Maker;
 
-	require 5.005_62; use strict; use warnings;
-	
-	our $VERSION = '0.05.18';
-	
-	use Class::Maker::Basic::Handler::Attributes;
-	
-	use Class::Maker::Basic::Fields;
-	
-	use Exporter;
-	
-	use subs qw(class);
-	
-	our $DEBUG = 0;
-	
-	our $TRACE = ( \*STDOUT, \*STDERR )[ ($ENV{CLASSMAKER_TRACE}||2) - 1 ];
-	
-	our %EXPORT_TAGS = ( 'all' => [ qw(class) ] );
-	
-	our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
-	
-	our @EXPORT = ();
-	
-	our @ISA = qw( Exporter );
-	
-	our $pkg = '<undefined class>';
-	
-	our $cpkg = $pkg;
-	
-	our $explicit = 0;
-	
-	# Preloaded methods go here.
-	
-	sub import
+require 5.005_62;
+
+use strict;
+
+use warnings;
+
+use attributes ();
+
+use Carp;
+
+use Class::Maker::Basic::Constructor;
+
+use Class::Maker::Basic::Handler::Attributes;
+
+use Class::Maker::Basic::Fields;
+
+use Class::Maker::Basic::Reflection qw(reflect classes);
+
+use Exporter;
+
+use subs qw(class);
+
+our $VERSION = '0.01';
+
+our $DEBUG = 0;
+
+	# read the perldoc Sub::Quotelike why i use , 'qa'
+
+our %EXPORT_TAGS = ( 'all' => [ qw(class reflect classes) ] );
+
+our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
+
+our @EXPORT = qw(class);
+
+our @ISA = qw( Exporter );
+
+our $pkg = '<undefined class>';
+
+our $cpkg = $pkg;
+
+our $explicit = 0;
+
+# Preloaded methods go here.
+
+sub import
+{
+	Class::Maker->export_to_level( 1, @_ );
+}
+
+sub class
+{
+	class_import( scalar caller, @_ );
+}
+
+sub class_import
+{
+		# $class is the caller package
+
+	my ( $class, @args ) = @_;
+
+	return unless @args;
+
+		# construct the destination package for the classes:
+		#
+		#	- we create the class within the current package (default)
+		#	- or create it in the current package
+		#	- or when starting with 'main::' or '::' we create it with the main package
+
+	unless( ref $args[0]  )
 	{
-		Class::Maker->export_to_level( 1, @_ );
+		$pkg = ( $args[0] =~ s/^(?:main)?::// ) ? $args[0] : $class.'::'.$args[0];
 	}
-	
-	sub class
+	else
 	{
-		class_import( scalar caller, @_ );
+			# We had no explicit destination package, so create the class in the current package
+
+		$pkg = $class;
 	}
-	
-	sub class_import
+
+		#remember caller package
+
+	$cpkg = $class;
+
+	carp "Package: $pkg (caller $class)" if $DEBUG;
+
+		# init class 'cause somebody could give an empty parameter
+		# list for abstract classes
+
+	Class::Maker::Basic::Fields::isa( [] );
+
+	Class::Maker::Basic::Fields::configure( { ctor => 'new', dtor => 'delete' } );
+
+	foreach my $arg ( @args )
 	{
-			# $class is the caller package
-	
-		my ( $class, @args ) = @_;
-	
-		return unless @args;
-	
-			# construct the destination package for the classes:
-			#
-			#	- we create the class within the current package (default)
-			#	- or create it in the current package
-			#	- or when starting with 'main::' or '::' we create it with the main package
-	
-		unless( ref $args[0]  )
+		carp "Import: $arg" if $DEBUG;
+
+		if( ref($arg) eq 'HASH' )
 		{
-			$pkg = ( $args[0] =~ s/^(?:main)?::// ) ? $args[0] : $class.'::'.$args[0];
-		}
-		else
-		{
-				# We had no explicit destination package, so create the class in the current package
-	
-			$pkg = $class;
-		}
-	
-			#remember caller package
-	
-		$cpkg = $class;
-	
-			# init class 'cause somebody could give an empty parameter
-			# list for abstract classes
-	
-		Class::Maker::Basic::Fields::isa( [] );
-	
-		Class::Maker::Basic::Fields::configure( { ctor => 'new', dtor => 'delete' } );
-	
-		foreach my $arg ( @args )
-		{
-			if( ref($arg) eq 'HASH' )
+			no strict 'refs';
+
+				# class reflection
+
+			${ Class::Maker::findclass( $pkg ) } = $arg;
+
+			foreach my $func ( sort { $b cmp $a } keys %$arg )
 			{
-				no strict 'refs';
-	
-				Class::Maker::Reflection::install( $arg );
-	
-				foreach my $func ( sort { $b cmp $a } keys %$arg )
+					# "classfields" for the class attributes/isa/configure/..
+
+				"Class::Maker::Basic::Fields::${func}"->( $arg->{$func} );
+			}
+		}
+	}
+}
+
+sub _get_code
+{
+	my $type = shift;
+
+	my $name = shift;
+
+	$name = "${pkg}::$name" if $explicit;
+
+	my $code =
+	{
+		new => \&Class::Maker::Basic::Constructor::new,
+
+		simple_new => \&Class::Maker::Basic::Constructor::simple_new,
+
+		debug_verbose => sub
+		{
+			carp "'$name' works..." if $DEBUG;
+		},
+
+		default => sub : lvalue
+		{
+			my $this = shift;
+
+		@_ ? $this->{$name} = shift : $this->{$name};
+		},
+
+		array => sub
+		{
+			my $this = shift;
+
+				$this->{$name} = [] unless exists $this->{$name};
+
+				@{ $this->{$name} } = () if @_;
+
+				foreach ( @_ )
 				{
-						# fields for the class attributes/isa/configure/..
-	
-					"Class::Maker::Basic::Fields::${func}"->( $arg->{$func}, $arg );
+					push @{ $this->{$name} }, ref($_) eq 'ARRAY' ? @{ $_ } : $_;
 				}
-			}
-		}
-	}
-	
-	sub _make_method
-	{
-		no strict 'refs';
-	
-		my $type = shift;
-	
-		my $name = shift;
-	
-			$Class::Maker::Basic::Handler::Attributes::name = $explicit ? "${pkg}::$name" : $name;
-	
-			no strict 'refs';
-	
-			if( *{ "Class::Maker::Basic::Handler::Attributes::${type}" }{CODE} )
-			{
-				return *{ "${pkg}::$name" } = Class::Maker::Basic::Handler::Attributes->$type;
-			}
-	
-	return *{ "${pkg}::$name" } = Class::Maker::Basic::Handler::Attributes->default;
-	}
 
-#
-# Reflection
-#
+		return wantarray ? @{$this->{$name}} : $this->{$name};
+		},
 
-package Class::Maker::Reflex;		# returned by Class::Maker::Reflection::reflect
-
-	sub definition : method
-	{
-		my $this = shift;
-
-	return $this->{def};
-	}
-	
-	sub parents : method
-	{
-		my $this = shift;
-
-		return unless exists $this->{isa};	
-				
-	return Class::Maker::Reflection::inheritance_isa( @{ $this->{isa} } );
-	}
-
-package Class::Maker::Reflection;
-
-	our $DEBUG = $Class::Maker::DEBUG;
-	
-		# DEEP : Whether reflect should traverse the @ISA tree and return all parent reflex's
-	
-	our $DEEP = 0;
-	
-	our $DEFINITION = 'CLASS';
-	
-	sub _get_definition
-	{
-		my $class = shift;
-	
-			no warnings;
-	
-			no strict 'refs';
-	
-	return \${ "${class}::".$Class::Maker::Reflection::DEFINITION };
-	}
-	
-	sub _get_isa
-	{
-		no strict 'refs';
-	
-	return @{ $_[0].'::ISA'};
-	}
-	
-	sub install
-	{
-		${ Class::Maker::Reflection::_get_definition( $pkg ) } = $_[0];
-	}
-	
-	sub reflect
-	{
-		my $class = ref( $_[0] ) || $_[0] || die;
-	
-			my $rfx = bless {  name => $class  }, 'Class::Maker::Reflex';
-	
-				# - First get the "${$DEFINITION}" href containing the class definition
-				# - find the functions of that class declerated with ': method'
-				# - catch up the parent class reflection if DEEP is activated
-				# - update "${$DEFINITION}"->{isa} with its real @ISA
-	
-			$rfx->{def} = ${ Class::Maker::Reflection::_get_definition( $class ) };
-	
-			$rfx->{methods} = find_methods( $rfx->{name} );
-	
-			no strict 'refs';
-	
-			if( $DEEP && defined *{ "${class}::ISA" }{ARRAY} )
-			{
-				$rfx->{isa} = \@{ *{ "${class}::ISA" }{ARRAY} };
-	
-				$rfx->{parents}->{$_} = reflect( $_ ) for @{ $rfx->{isa} };
-			}
-	
-	return $rfx;
-	}
-	
-	sub classes
-	{
-		no strict 'refs';
-	
-		my @found;
-	
-		my $path = shift if @_ > 1;
-	
-		foreach my $pkg ( @_ )
+		hash => sub
 		{
-			next unless $pkg =~ /::$/;
-	
-			$path .= $pkg;
-	
-			if( $path =~ /(.*)::$/ )
-			{
-				my $clean_path = $1;
-	
-				if( $path ne 'main::' )
+				my $this = shift;
+
+				unless( exists $this->{$name} )
 				{
-					if( my $href_cls = reflect( $clean_path ) )
-					{
-						push @found, { $clean_path => $href_cls };
-					}
+					$this->{$name} = {};
 				}
-	
-				foreach my $symbol ( sort keys %{$path} )
+
+				foreach my $href ( @_ )
 				{
-					if( $symbol =~ /::$/ && $symbol ne 'main::' )
+					if( ref($href) eq 'HASH' )
 					{
-						push @found,  classes( $path, $symbol );
-					}
-				}
-			}
-		}
-	
-	return @found;
-	}
-	
-	use attributes;
-	
-	sub find_methods
-	{
-		my $class = shift;
-	
-			my $methods = [];
-	
-			no strict 'refs';
-	
-			foreach my $pkg ( $class.'::' )
-			{
-				foreach ( sort keys %{$pkg} )
-				{
-					unless( /::$/ )
-					{
-						if( defined *{ "$pkg$_" }{CODE} )
+						foreach my $key ( keys %{ $href } )
 						{
-							if( my $type = attributes::get( \&{ "$pkg$_" } ) )
-							{
-								push @$methods, "$_" if $type =~ /method/i;
-							}
+							$this->{$name}->{$key} = $href->{$key};
 						}
 					}
 				}
-			}
-	
-	return $methods;
-	}
-	
-	sub find
-	{
-		my %request = @_;
-	
-		my @result;
-	
-				# parsing all references in a package (via symbol table)
-	
-			while( my ( $where, $what ) = each %request )
-			{
-				no strict 'refs';
-	
-				foreach my $pkg ( $where.'::' )
-				{
-					print $Class::Maker::TRACE "Searching in package '$where' for '$what' instances\n" if $DEBUG;
-	
-					foreach ( sort keys %{$pkg} )
-					{
-						unless( /::$/ )
-						{
-							if( defined *{ "$pkg$_" } )
-							{
-								my $sref = \${ "$pkg$_" };
-	
-								if( ref( $sref ) eq 'REF' )
-								{
-									my $type = ref( $$sref );
-	
-									printf $Class::Maker::TRACE "%20s %10s %s isa($what)\n", '$'.$_, $type if $$sref->isa( $what ) and $DEBUG;
-	
-									push @result, $$sref;
-								}
-							}
-						}
-					}
-				}
-			}
-	
-	return \@result;
-	}
-	
-		# helpers
-	
-	sub _isa_tree
-	{
-		my $list = shift;
-	
-		my $level = shift;
-	
-		for my $child ( @_ )
+
+		return wantarray ? %{ $this->{$name} } : $this->{$name};
+		},
+	};
+
+return $code->{lc $type} || $code->{'default'};
+}
+
+sub _make_method
+{
+	no strict 'refs';
+
+	my $args = shift;
+
+	my $method_type = shift;
+
+		foreach my $name ( @$args )
 		{
-			my @parents = Class::Maker::Reflection::_get_isa( $child );
-	
-			$level++;
-	
-			push @{ $list->{$level} }, $child;
-	
-			warn sprintf "\@%s::ISA = qw(%s);",$child , join( ' ', @parents ) if $Class::Maker::DEBUG;
-	
-			_isa_tree( $list, $level, @parents );
-	
-			$level--;
+			my $varname = "${pkg}::$name";
+
+			carp "func: $varname\n" if $DEBUG;
+
+			*{ $varname } = _get_code( $method_type, $name ) unless defined *{ $varname }{CODE};
 		}
-	}
-	
-		# returns the isa tree sorted by level of recursion
-	
-		# 5 -> Exporter
-		# 4 -> Object::Debugable
-		# 3 -> Person, Exporter
-		# 2 -> Employee, Exporter, Object::Debugable
-		# 1 -> Doctor
-	
-	sub isa_tree
-	{
-		my $list = {};
-	
-		_isa_tree( $list, 0, @_ );
-	
-	return $list;
-	}
-	
-		# returns the isa tree in a planar list (for con-/destructor queue's)
-	
-	sub inheritance_isa
-	{
-		warn sprintf "SCANNING ISA FOR (%s);", join( ', ', @_ ) if $Class::Maker::DEBUG;
-	
-		my $construct_list = isa_tree( @_ );
-	
-		my @ALL;
-	
-		foreach my $level ( sort { $b <=> $a } keys %$construct_list )
-		{
-			push @ALL, @{ $construct_list->{$level} };
-		}
-	
-	return \@ALL;
-	}
+}
+
+sub findclass
+{
+	my $class = shift;
+
+		no warnings;
+
+		no strict 'refs';
+
+return \${ "${class}::CLASS" };
+}
 
 1;
-
 __END__
 
 =head1 NAME
@@ -387,12 +231,14 @@ Class::Maker - classes, reflection, schema, serialization, attribute- and multip
 
 use Class::Maker qw(class);
 
-class Human,
+class UnixUser,
 {
-	isa => [qw( ParentClass )],
+	isa => [qw( .ParentClassA ParentClassB )],
 
-	public =>
+	attribute =>
 	{
+		int		=> [qw(uid gid)],
+
 		string	=> [qw(name lastname)],
 
 		ref		=> [qw(father mother)],
@@ -400,50 +246,35 @@ class Human,
 		array	=> [qw(friends)],
 
 		custom	=> [qw(anything)],
-	},
-
-	private =>
-	{
-		int     => [qw(dummy1 dummy2)],
-	},
+	}
 
 	configure =>
 	{
-		ctor => 'create',
+		# Future: also allow: ctor or dtor => $coderef
+
+		ctor => 'makenew',
 
 		explicit => 0,
 	},
 };
 
-sub Human::greeting : method
+sub UnixUser::hello : method
 {
 	my $this = shift;
 
 		printf 'This is %s (%d)', $this->name, $this->uid;
+
+return 1;
 }
 
-class UnixUser,
-{
-	isa => [qw( Human )],
+	my $a = UnixUser->new( uid => 1, gid => 2, name => Murat );
 
-	public =>
-	{
-		int		=> [qw(uid gid)],
+	$a->father( UnixUser->new( name => Suleyman ) );
 
-		string	=> [qw(username)],
-	},
-};
-
-	my $a = Human->new( uid => 1, gid => 2, name => Bart );
-
-	$a->father( Human->new( name => Houmer ) );
-
-	$a->greeting();
+	$a->hello();
 
 	$a->uid = 12;
 
-	$a->_dummy1( 'bla' );
-
 =head1 DESCRIPTION
 
 Class::Maker introduces the concept of classes via a "class" function. It automatically creates packages, ISA, new and attribute-handlers. The classes can inherit from common perl-classes and class-maker classes. Single and multiple inheritance is supported.
@@ -488,7 +319,7 @@ class Person,
 {
 	isa => [ Something ],
 
-	public =>
+	attributes =>
 	{
 		scalar => [qw( name age )],
 	},
@@ -496,6 +327,37 @@ class Person,
 
 When using "class", you do not explictly need "package". The function does all symbol creation for you. It is more a class decleration (like in java/cpp/..):
 
+=head1 INTERNALS
+
+Following happens in the background, when using 'class':
+
+=over 4
+
+=item creates a package "Person".
+=item sets @Person::ISA to the [ 'Something' ].
+=item creates method handlers for the attributes (including lvalue methods).
+=over 4
+while only "hash" and "array" keys are keywords, any other
+will result in a simple get/set method (this is very usefull for pseudo types, wich can be later
+implemented).
+
+=back
+
+=item exports a default constructor "Person::new()"
+=item which handles argument initialization
+=item has a mechanism for initializing the parent objects (including MI)
+=item so you don't need to do it yourself
+=item creates $Person::CLASS holding a hashref to the structure [the second argument]
+=item good for reflection: i.e. you can get runtime information about the class
+=over 4
+=item usable for dependency/class walking
+=item creating on-the-fly persistance => in combination with Tangram (through reflection)
+=item it creates the complete tangram schema tree (Tangram users know how hard it is
+=item to have x hand-maintained schema-hashes => here you get everything automatic).
+=back
+
+=back
+
 =head1 CLASS FUNCTION
 
 The 'class' function is very central to Class::Maker. It may be called with or without braces.
@@ -587,14 +449,14 @@ Fields are the keys in the hashref given as the second (or first if the first ar
 
 =item private
 
-	All properties in the 'private' section, get a '_' prepended to their names.
+	All properties in the 'private' section, get a '__' prepended to their names.
 
 	private =>
 	{
 		int		=> [qw(uid gid)],
 	},
 
-	So you must access 'uid' with $obj->_uid();
+	So you must access 'uid' with $obj->__uid();
 
 =item public|attribute|attr
 
@@ -613,7 +475,7 @@ Fields are the keys in the hashref given as the second (or first if the first ar
 
 =item configure
 
-	This Field is for general options. Basicly following options are supported:
+	This Field is for general options. In the basic package following options are supported:
 
 		a) new: The name of the default constructor is 'new'. With this option you can change the
 		name to something of your choice. For instance:
@@ -634,7 +496,6 @@ Fields are the keys in the hashref given as the second (or first if the first ar
 		key name:
 
 		Example:
-
 		'A' inherits 'B'. Both have a 'name' property. With explicit internally the fields
 		are distinct:
 
@@ -647,23 +508,6 @@ Fields are the keys in the hashref given as the second (or first if the first ar
 		Only use this feature, if you have fear that name clashes could appear, beside overloading. Per default
 		it is turned off, because i suppose that most class designers care for name clashes themselfs.
 
-		c) I<private>
-
-			- prefix string (default '_') for private functions can be changed with this.
-
-		Example:
-
-			private =>
-			{
-				int => [qw(dummy1)],
-			},
-			configure =>
-			{
-				private => { prefix => '__' },
-			},
-
-		would force to access 'dummy1' via ->__dummy1().
-
 =item automethod
 
 	Reserved.
@@ -718,37 +562,6 @@ Fields are the keys in the hashref given as the second (or first if the first ar
 
 =back
 
-=head1 INTERNALS
-
-Following happens in the background, when using 'class':
-
-=over 4
-
-=item creates a package "Person".
-=item sets @Person::ISA to the [ 'Something' ].
-=item creates method handlers for the attributes (including lvalue methods).
-=over 4
-while only "hash" and "array" keys are keywords, any other
-will result in a simple get/set method (this is very usefull for pseudo types, wich can be later
-implemented).
-
-=back
-
-=item exports a default constructor "Person::new()"
-=item which handles argument initialization
-=item has a mechanism for initializing the parent objects (including MI)
-=item so you don't need to do it yourself
-=item creates $Person::CLASS holding a hashref to the structure [the second argument]
-=item good for reflection: i.e. you can get runtime information about the class
-=over 4
-=item usable for dependency/class walking
-=item creating on-the-fly persistance => in combination with Tangram (through reflection)
-=item it creates the complete tangram schema tree (Tangram users know how hard it is
-=item to have x hand-maintained schema-hashes => here you get everything automatic).
-=back
-
-=back
-
 =head2 PERFORMANCE
 
 I never benchmarked Class::Maker. Because the internal representation is just the same as for
@@ -768,19 +581,10 @@ obligat:	qw(class)
 
 class by default.
 
-=head1 KNOWN BUGS/PROBLEMS
-
-	* isa => [qw( )] isnt in sync with @ISA. When @ISA (or isa) is modified after initation, the $reflex->{isa} will only
-	represent the state during object initiation.
-
 =head1 AUTHOR
 
 Author: Murat Uenalan (muenalan@cpan.org)
 
-Contributions (Ideas or Code):
-
-	- Terrence Brannon
-
 =head1 COPYRIGHT
 
 (c) 2001 by Murat Uenalan. All rights reserved.
