@@ -1,5 +1,5 @@
 #
-# Author:	Murat Uenalan (muenalan@cpan.org)
+# Author:	Murat Uenalan (murat.uenalan@charite.de)
 #
 # Copyright:	Copyright (c) 1997 Murat Uenalan. All rights reserved.
 #
@@ -11,13 +11,21 @@ package Class::Maker::Basic::Constructor;
 
 	require 5.005_62; use strict; use warnings;
 
-use Exporter;
+	#use Bugfix::ref qw(ref bless);
 
-our $VERSION = '0.02';
+	use Carp;
+
+	use IO::Extended ':all';
+
+	use Exporter;
+
+our $VERSION = '0.01';
+
+our $DEBUG = 0;
 
 our @ISA = qw(Exporter);
 
-our %EXPORT_TAGS = ( 'all' => [ qw(new) ],	'std' => [ qw(new) ] );
+our %EXPORT_TAGS = ( 'all' => [ qw(new defaults) ],	'std' => [ qw(new) ] );
 
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 
@@ -25,8 +33,6 @@ our @EXPORT = qw();
 
 # Preloaded methods go here.
 
-our @init_methods = qw( init initialize );
-
 sub simple_new
 {
 	my $this = shift;
@@ -40,72 +46,42 @@ sub simple_new
 	gets its classname prepended like "Object::attribute" into the hash
 	representation of the object.
 
-	But you must be aware that when initializing via new( public => ),
+	But you must be aware that when initializing via new( attribute => ),
 	alwas the first parent attribute is used for the initalization.
 
 	new( Parent1::field => 'bla', Parent2::field => 'blabla' );
 
 =cut
 
-	# multiple inheritance constructor (shouldn't be overriden, otherwise no MI)
+	# multiple inheritance constructor
 
 sub new
 {
-	my $what = shift;
+	my $class = shift;
 
-		my $class = ref( $what ) || $what;
-
-			# convert constructor arguments to accessor/method calls
-
-		my @args = @_;
-		
-		my %args;
-		
-		if( $class->can( '_arginit' ) )
-		{
-			$class->_arginit( \@args );
-		}
-
-		%args = @args;
-		
-		my $args = \%args;
-
-		_filter_argnames( $args );
-
-			# look if we just need cloning
-
-		if( ref( $what ) )
-		{
-			my %copy = %$what;
-
-			my $clone = bless \%copy, $class;
-
-			while( my ( $key, $value ) = each %args )
-			{
-				$clone->$key( $value );
-			}
-
-			return $clone;
-		}
-
-			# if we do not clone, construct a new instance
+		$class = ref( $class ) || $class;
 
 		my $this = bless {}, $class;
 
-			# preset all defaults
+		my %temp = @_;
 
-		my $rfx = Class::Maker::Reflection::reflect( $class ) or die;
-
-		if( $rfx->definition )
+		foreach my $key ( keys %temp )
 		{
-			_defaults( $this, $rfx->definition->{default} ) if exists $rfx->definition->{default};
+			if( $key =~ /^\-+(.*)/ )
+			{
+				$temp{$1} = $temp{$key};
+
+				delete $temp{$key};
+			}
 		}
 
-			# inheriting attributes here
+		my $args = \%temp;
 
-		warn( sprintf "NEW TRAVERSING ISA: %s", join( ', ', @{ Class::Maker::Reflection::inheritance_isa( ref( $this ) ) } ) ) if $Class::Maker::DEBUG;
+		# inheriting attributes here
 
-		foreach my $parent ( @{ Class::Maker::Reflection::inheritance_isa( ref( $this ) || die ) } )
+		carp sprintf "NEW TRAVERSING ISA: %s", join( ', ', @{ inheritance_isa( ref( $this ) ) } ) if $DEBUG;
+
+		foreach my $parent ( @{ inheritance_isa( ref( $this ) || die ) } )
 		{
 			my $class = ref($this);
 
@@ -113,13 +89,16 @@ sub new
 
 			no strict 'refs';
 
-			"${parent}::_preinit"->( $this, $args ) if defined *{ "${parent}::_preinit" }{CODE};
-
-			foreach my $init_method ( @init_methods )
+			if( defined *{ "${parent}::_preinit" }{CODE} )
 			{
-				if( defined *{ "${parent}::${init_method}" }{CODE} )
+				"${parent}::_preinit"->( $this, $args );
+			}
+
+			foreach my $initmethod ( qw( init initialize ) )
+			{
+				if( defined *{ "${parent}::${initmethod}" }{CODE} )
 				{
-					"${parent}::${init_method}"->( $this, $args );
+					"${parent}::${initmethod}"->( $this, $args );
 
 					last;
 				}
@@ -135,59 +114,105 @@ sub new
 				}
 			}
 
-			"${parent}::_postinit"->( $this, $args ) if defined *{ "${parent}::_postinit" }{CODE};
+			if( defined *{ "${parent}::_postinit" }{CODE} )
+			{
+				"${parent}::_postinit"->( $this, $args );
+			}
 
 			bless $this, $class;
 		}
 
 		# call constructor arguments as functions, because we assume attribute-handlers
 
-		warn "Unhandled new() arg: '$_' (Implement attribute-handler or check spelling)" for keys %args;
+		foreach ( keys %temp )
+		{
+			warn "Unhandled ctor arg: '$_' (Implement attribute-handler or check spelling)";
+		}
 
 return $this;
 }
 
 # functions
 
-sub _filter_argnames
-{
-	my $temp = shift;
-
-			# rename all -arg or --arg fields
-
-		foreach my $key ( keys %$temp )
-		{
-			if( $key =~ /^\-+(.*)/ )
-			{
-				$temp->{$1} = $temp->{$key};
-
-				delete $temp->{$key};
-			}
-		}
-}
-
-sub _defaults
+sub defaults
 {
 	my $this = shift;
 
-	my $args = shift;
+	my %args = @_;
 
-	no strict 'refs';
-
-	foreach my $attr ( keys %$args )
+	foreach my $attr ( keys %args )
 	{
 		if( my $coderef = $this->can( $attr ) )
 		{
-			print "Setting $this default (via coderef $coderef) $attr = ", $args->{$attr}, "\n" if $Class::Maker::DEBUG;
-
-			$coderef->( $this, $args->{$attr} );
-
-			#$this->$attr( $args->{$attr} );
-			#$this->{$attr} = $args->{$attr};
+			$this->$coderef( $args{$attr} );
 		}
 	}
 }
 
+sub classISA
+{
+	no strict 'refs';
+
+return @{ $_[0].'::ISA'};
+}
+
+sub _isa_tree
+{
+	my $list = shift;
+
+	my $level = shift;
+
+	for my $child ( @_ )
+	{
+		my @parents = classISA( $child );
+
+		$level++;
+
+		push @{ $list->{$level} }, $child;
+
+		carp sprintf "\@%s::ISA = qw(%s);",$child , join( ' ', @parents ) if $DEBUG;
+
+		_isa_tree( $list, $level, @parents );
+
+		$level--;
+	}
+}
+
+	# returns the isa tree sorted by level of recursion
+
+	# 5 -> Exporter
+	# 4 -> Object::Debugable
+	# 3 -> Person, Exporter
+	# 2 -> Employee, Exporter, Object::Debugable
+	# 1 -> Doctor
+
+sub isa_tree
+{
+	my $list = {};
+
+	_isa_tree( $list, 0, @_ );
+
+return $list;
+}
+
+	# returns the isa tree in a planar list (for con-/destructor queue's)
+
+sub inheritance_isa
+{
+	carp sprintf "SCANNING ISA FOR (%s);", join( ', ', @_ ) if $DEBUG;
+
+	my $construct_list = isa_tree( @_ );
+
+	my @ALL;
+
+	foreach my $level ( sort { $b <=> $a } keys %$construct_list )
+	{
+		push @ALL, @{ $construct_list->{$level} };
+	}
+
+return \@ALL;
+}
+
 1;
 
 __END__
