@@ -4,19 +4,19 @@
 
 package Class::Maker::Basic::Fields;
 
+use Carp;
+
 sub configure
 {
 	my $args = shift;
 
-	my $reflex = shift;
-
-	Class::Maker::_make_method( 'new', exists $args->{ctor} ? $args->{ctor} : 'new' );
+	Class::Maker::_make_method( [$args->{ctor}], 'new' ) if exists $args->{ctor};
 
 	if( exists $args->{explicit} )
 	{
-		$Class::Maker::explicit = $args->{explicit};
+		$explicit = $args->{explicit};
 
-		warn "EXPLICIT $explicit" if $DEBUG;
+		carp "EXPLICIT $explicit" if $DEBUG;
 	}
 
 	# dtor is missing here
@@ -26,19 +26,6 @@ sub persistance
 {
 	my $args = shift;
 
-	my $reflex = shift;
-
-	no strict 'refs';
-}
-
-       # simply do nothing and store it for Data::Type::Guard
-
-sub types
-{
-	my $args = shift;
-
-	my $reflex = shift;
-
 	no strict 'refs';
 }
 
@@ -46,8 +33,6 @@ sub isa
 {
 	my $args = shift;
 
-	my $reflex = shift;
-
 		no strict 'refs';
 
 			# Transform parent classes
@@ -67,8 +52,6 @@ sub version
 {
 	my $args = shift;
 
-	my $reflex = shift;
-
 	no strict 'refs';
 
 	${ "${Class::Maker::pkg}::VERSION" } = shift @$args;
@@ -89,59 +72,64 @@ sub default
 {
 	my $args = shift;
 
-	my $reflex = shift;
-
 	foreach my $attr ( keys %$args )
 	{
-		warn sprintf "\tpredefined default %s = '%s'\n", $attr, $args->{$attr} if $DEBUG;
+		carp "\tsetting default for $attr..\n" if $DEBUG;
 	}
 }
 
-our $protected_prefix = { public => '', protected => '__', private => '_' };
-
-sub _create_accessor
+sub attribute
 {
-	my $protected = shift;
-
 	my $args = shift;
 
-	my $reflex = shift;
+	foreach my $type ( keys %$args )
+	{
+		carp "\tkey: $type\n" if $DEBUG;
 
-		foreach my $type ( keys %$args )
+			# check for non href and take the keys only
+
+		if( ref( $args->{$type} ) eq 'HASH' )
 		{
-			my @attributes = ( ref( $args->{$type} ) eq 'HASH' ) ? keys %{$args->{$type}} : @{ $args->{$type} };
+			Class::Maker::_make_method( [(keys %{$args->{$type}})], $type ) if keys %{$args->{$type}};
 
-			foreach my $name ( @attributes )
-			{
-				Class::Maker::_make_method( $type, $protected_prefix->{$protected}.$name );
-			}
+			#print "attribute HASH listing detected\n";
 		}
+		else
+		{
+			Class::Maker::_make_method( $args->{$type}, $type ) if scalar @{ $args->{$type} };
+		}
+	}
+}
+
+sub attr
+{
+	attribute( @_ );
 }
 
 sub public
 {
-	_create_accessor( 'public', @_ );
+	attribute( @_ );
 }
 
-sub private
+sub privat
 {
-	_create_accessor( 'private', @_ );
+	attribute( @_ );
 }
 
-sub protected
+sub automethod
 {
-	_create_accessor( 'protected', @_ );
+	my $args = shift;
+
+	carp join( ',', @$args ), "\n" if $DEBUG;
 }
 
 sub has
 {
 	my $args = shift;
 
-	my $reflex = shift;
-
 	foreach ( keys %$args )
 	{
-		warn "\tkey: $_\n" if $DEBUG;
+		carp "\tkey: $_\n" if $DEBUG;
 	}
 }
 
