@@ -4,9 +4,7 @@ use Class::Maker::Examples::Human;
 
 #use Object::Debugable;
 
-package  Employee;
-
-Class::Maker::class
+Class::Maker::class Employee,
 {
 	version => $VERSION,
 
@@ -14,16 +12,11 @@ Class::Maker::class
 
 	#has => { Person => [qw(father mother sister brother)],
 
-	public =>
+	attribute =>
 	{
 		getset => [qw( firstname income payment position )],
 	},
 
-	private =>
-	{
-		int => [qw( dummy1 dummy2 )],
-	},
-
 	configure =>
 	{
 		ctor => 'new',
@@ -31,19 +24,17 @@ Class::Maker::class
 		dtor => 'delete',
 
 		explicit => 1,
-
-		private => { prefix => '__' },
 	},
 };
 
-sub _preinit
+sub Employee::_preinit
 {
 	my $this = shift;
 
 		#whereami();
 }
 
-sub _postinit
+sub Employee::_postinit
 {
 	my $this = shift;
 
@@ -54,10 +45,11 @@ sub _postinit
 
 =cut
 
-sub phantom : method
+sub Employee::phatom : method
 {
 	my $this = shift;
 
+
 return;
 }
 
