@@ -2,29 +2,23 @@ our $VERSION = '0.001';
 
 use Class::Maker::Examples::Human;
 
+Class::Maker::class 'Vehicle',
 {
-	package Vehicle;
-
-	Class::Maker::class
+	attribute =>
 	{
-		attribute =>
-		{
-			int => [qw( wheels )],
+		int => [qw( wheels )],
 
-			string => [qw( model )],
-		},
-	};
-}
+		string => [qw( model )],
+	},
+};
 
-package User;
-
-Class::Maker::class
+Class::Maker::class 'User',
 {
 	version => '0.01',
 
 	isa => [qw( Human )],
 
-	public =>
+	attribute =>
 	{
 		int => [qw( logins )],
 
@@ -38,16 +32,14 @@ Class::Maker::class
 	},
 };
 
-sub _preinit
+sub User::_preinit
 {
 	my $this = shift;
 
 			@$this{qw( lastlog registered )} = qw(NULL NULL);
 }
 
-package User::Group;
-
-Class::Maker::class
+Class::Maker::class 'User::Group',
 {
 	attribute =>
 	{
